import logging
import subprocess

from service_buddy.util.log_handler import print_red_bold


def invoke_process(args, exec_dir=None, dry_run=False):
    if dry_run:
        print_red_bold(u"\t {}".format(str(args)))
        return 0
    else:
        arg_list = {'args': args}
        if exec_dir:
            arg_list['cwd'] = exec_dir
        return subprocess.call(**arg_list)


import json
import os
import logging
from service_buddy.ci.bamboo_build_creator import BambooBuildCreator
from service_buddy.ci.travis_build_creator import TravisBuildCreator


class BuildCreator(object):
    def __init__(self, template_directory, dry_run):
        super(BuildCreator, self).__init__()
        default_path = os.path.join(template_directory, "build-config.json")
        if os.path.exists(default_path):
            with open(default_path) as fp:
                defaults = json.load(fp)
                self.default_provider = defaults.get('provider', None)
                self.build_templates = defaults.get('build-templates', {})
                self.default_config = defaults
        else:
            logging.warn("Could not local 'build-config.json' in code template directory")
            self.default_provider = "bamboo"
        self.code_creators = {
            BambooBuildCreator.get_type(): BambooBuildCreator(),
            TravisBuildCreator.get_type(): TravisBuildCreator()}
        if self.default_provider not in self.code_creators:
            raise Exception("Requested provider is not configured {}".format(self.default_provider))
        else:
            self._get_default_build_creator().init(dry_run=dry_run,
                                                   default_config=self.default_config,
                                                   build_templates=self.build_templates)

    def _get_default_build_creator(self):
        # type: () -> BuildCreator
        return self.code_creators[self.default_provider]

    def create_project(self, service_definition, app_dir):
        return self._get_default_build_creator().create_project(service_definition=service_definition,
                                                                app_dir=app_dir)



import os

import click
from infra_buddy.context.deploy_ctx import DeployContext

from service_buddy.code.code_creator import CodeCreator
from service_buddy.commandline import cli
from service_buddy.context.service_context import ServiceContext
from service_buddy.service.service import Service
from infra_buddy.commands.bootstrap import command as bootstrap


@cli.command(name='bootstrap', short_help="Create a repository containing the initial definition for a micro-service "
                                          "stack managed by service-buddy.")
@click.option("--application", envvar='APPLICATION', required=True,
              help='Application name for new micro-service stack.  Brevity suggested.')
@click.option("--deploy-region", envvar='REGION',
              help='Region to initialize the application.')
@click.option("--skip-infrastructure-bootstrap", is_flag=True, envvar="SKIP_ENV",
              help="Skip the bootstrap of the infrastructure for "
                   "this application.")
@click.pass_obj
def bootstrap(service_ctx, application, skip_infrastructure_bootstrap, deploy_region=None):
    # type: (ServiceContext, str,bool,str) -> None
    cc = CodeCreator("", service_ctx.dry_run)
    service_def = Service(app=application, role="master", definition={})
    service_def.set_service_type("service-buddy-master")
    directory = service_ctx.destination_directory
    if directory == "./code": #override default for this usecase
        directory = "./"
    cc.create_project(service_definition=service_def, app_dir=directory)
    if deploy_region:
        os.environ.setdefault('REGION', deploy_region)
    if not skip_infrastructure_bootstrap:
        bootstrap.do_command(
            DeployContext.create_deploy_context(
                application=application,
                role="none",
                environment='dev'
            ),
            ['ci', 'prod']
        )
